--
-- Enable CDC on the database
--
USE [DWDemoSource]
GO

EXEC sys.sp_cdc_enable_db 
GO

--
-- Enable CDC on the Clients table
--

EXEC sys.sp_cdc_enable_table
@source_schema = N'dbo',
@source_name   = N'SourceClient',
@role_name     = NULL,
@supports_net_changes = 1
GO

--
-- Get all changes
--
DECLARE @from_lsn binary(10), @to_lsn binary(10)
SET @from_lsn = sys.fn_cdc_get_min_lsn('dbo_SourceClient')
SET @to_lsn   = sys.fn_cdc_get_max_lsn()
SELECT * FROM cdc.fn_cdc_get_all_changes_dbo_SourceClient(@from_lsn, @to_lsn, N'all')
GO

--
-- Show LSN -> DateTime mapping
--
DECLARE @from_lsn binary(10), @to_lsn binary(10)
SET @from_lsn = sys.fn_cdc_get_min_lsn('dbo_SourceClient')
SET @to_lsn   = sys.fn_cdc_get_max_lsn()
SELECT sys.fn_cdc_map_lsn_to_time ( @from_lsn ) as "Start Date/Time", sys.fn_cdc_map_lsn_to_time ( @to_lsn ) as "End Date/Time"
GO


--
-- Wrapper generation
-- 
sys.sp_cdc_generate_wrapper_function
      @capture_instance = 'dbo_SourceClient'
    , @closed_high_end_point = 1

--
-- Create All Changes function
--
create function [fn_all_changes_dbo_SourceClient]   ( 
	@start_time datetime = null,    
	@end_time datetime = null,    
	@row_filter_option nvarchar(30) = N'all'   
)   returns @resultset table ( 
	[__CDC_STARTLSN] binary(10), 
	[__CDC_SEQVAL] binary(10), 
	[__CDC_OPERATION] varchar(2),
	[ClientId] int, 
	[ClientBusinessKey] nvarchar(20), 
	[FirstName] nvarchar(50), 
	[MiddleName] nvarchar(50), 
	[LastName] nvarchar(50), 
	[AddressLine1] nvarchar(120), 
	[AddressLine2] nvarchar(120), 
	[BirthDate] date, 
	[Gender] nvarchar(1), 
	[YearlyIncome] money, 
	[TotalChildren] tinyint, 
	[DateFirstPolicy] date, 
	[CommuteDistance] nvarchar(15), 
	[City] nvarchar(30), 
	[StateProvinceCode] nvarchar(3), 
	[StateProvinceName] nvarchar(50), 
	[CountryRegionCode] nvarchar(3), 
	[PostalCode] nvarchar(15)	     
) as   begin    
	declare @from_lsn binary(10), @to_lsn binary(10)         
	if (@start_time is null)     
		select @from_lsn = [sys].[fn_cdc_get_min_lsn]('dbo_SourceClient')    
	else    
		begin     
			if ([sys].[fn_cdc_map_lsn_to_time]([sys].[fn_cdc_get_min_lsn]('dbo_SourceClient')) > @start_time) or        
			   ([sys].[fn_cdc_map_lsn_to_time]([sys].[fn_cdc_get_max_lsn]()) < @start_time)      
				select @from_lsn = null     
			else      
				select @from_lsn = [sys].[fn_cdc_increment_lsn]([sys].[fn_cdc_map_time_to_lsn]('largest less than or equal',@start_time))    
		end        
		if (@end_time is null)     
			select @to_lsn = [sys].[fn_cdc_get_max_lsn]()   
		else    
			begin 
				if [sys].[fn_cdc_map_lsn_to_time]([sys].[fn_cdc_get_max_lsn]()) < @end_time      
					select @to_lsn = null     
				else      
					select @to_lsn = [sys].[fn_cdc_map_time_to_lsn]('largest less than or equal',@end_time)    
			end        
				if @from_lsn is not null and @to_lsn is not null and (@from_lsn = [sys].[fn_cdc_increment_lsn](@to_lsn))     
				   return insert into @resultset  
						select [__$start_lsn] as [__CDC_STARTLSN],
							   [__$seqval] as [__CDC_SEQVAL], 
							   case [__$operation]
								when 1 then 'D'
								when 2 then 'I'
								when 3 then 'UO'
								when 4 then 'UN'
								when 5 then 'M'
								else null
							   end as [__CDC_OPERATION],							   
							   [ClientId], 
							   [ClientBusinessKey], 
							   [FirstName], 
							   [MiddleName], 
							   [LastName], 
							   [AddressLine1], 
							   [AddressLine2], 
							   [BirthDate], 
							   [Gender], 
							   [YearlyIncome], 
							   [TotalChildren], 
							   [DateFirstPolicy], 
							   [CommuteDistance], 
							   [City], 
							   [StateProvinceCode], 
							   [StateProvinceName], 
							   [CountryRegionCode], 
							   [PostalCode]
						from [cdc].[fn_cdc_get_all_changes_dbo_SourceClient](@from_lsn, @to_lsn, @row_filter_option) 
						order by [__$seqval], [__$operation]        
						return   
end

--
-- Execute wrapper function
--

DECLARE @from datetime, @to datetime
SET @from = (SELECT sys.fn_cdc_map_lsn_to_time(sys.fn_cdc_get_min_lsn('dbo_SourceClient')))
SET @to = (SELECT sys.fn_cdc_map_lsn_to_time( sys.fn_cdc_get_max_lsn()))
SELECT * FROM [dbo].[fn_all_changes_dbo_SourceClient] (
   @from
  ,@to
  ,N'all')
GO


---
--- Disable CDC
---

use [DWDemoSource]
GO
sys.sp_cdc_disable_db
use [master]
GO