USE [master]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

IF  EXISTS (SELECT name FROM sys.databases WHERE name = N'DWDemo')
DROP DATABASE [DWDemo]
GO

USE [master]
GO

CREATE DATABASE [DWDemo]
GO

USE [DWDemo]
GO

IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DimClaimCode]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DimClaimCode](
	[ClaimCodeKey] [nvarchar](4) NOT NULL,
	[Description] [ntext] NULL,
 CONSTRAINT [PK_DimClaimCode] PRIMARY KEY CLUSTERED 
(
	[ClaimCodeKey] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
END
GO

IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DimClient]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DimClient](
	[ClientId] [int] IDENTITY(1,1) NOT NULL,
	[ClientBusinessKey] [nvarchar](20) NOT NULL,
	[FirstName] [nvarchar](50) NOT NULL,
	[MiddleName] [nvarchar](50) NULL,
	[LastName] [nvarchar](50) NULL,
	[AddressLine1] [nvarchar](120) NULL,
	[AddressLine2] [nvarchar](120) NULL,
	[BirthDate] [date] NULL,
	[Gender] [nvarchar](1) NULL,
	[YearlyIncome] [money] NULL,
	[TotalChildren] [tinyint] NULL,
	[DateFirstPolicy] [date] NULL,
	[CommuteDistance] [nvarchar](15) NULL,
	[City] [nvarchar](30) NULL,
	[StateProvinceCode] [nvarchar](3) NULL,
	[StateProvinceName] [nvarchar](50) NULL,
	[CountryRegionCode] [nvarchar](3) NULL,
	[PostalCode] [nvarchar](15) NULL,
	[StartDate] [date] NOT NULL,
	[EndDate] [date] NULL,
 CONSTRAINT [PK_DimClient] PRIMARY KEY CLUSTERED 
(
	[ClientId] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
END
GO

IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DimPolicyType]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DimPolicyType](
	[PolicyTypeKey] [nvarchar](4) NOT NULL,
	[Description] [ntext] NULL,
 CONSTRAINT [PK_DimPolicyType] PRIMARY KEY CLUSTERED 
(
	[PolicyTypeKey] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
END
GO

IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DimPolicy]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[DimPolicy](
	[PolicyKey] [int] IDENTITY(1,1) NOT NULL,
	[ClientId] [int] NOT NULL,
	[Deductable] [money] NULL,
	[Maximum] [money] NULL,
	[PolicyTypeKey] [nvarchar](4) NOT NULL,
 CONSTRAINT [PK_FactPolicies] PRIMARY KEY CLUSTERED 
(
	[PolicyKey] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
END
GO

IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DimPolicy_DimClient]') AND parent_object_id = OBJECT_ID(N'[dbo].[DimPolicy]'))
ALTER TABLE [dbo].[DimPolicy]  WITH CHECK ADD  CONSTRAINT [FK_DimPolicy_DimClient] FOREIGN KEY([ClientId])
REFERENCES [dbo].[DimClient] ([ClientId])
GO

IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DimPolicy_DimClient]') AND parent_object_id = OBJECT_ID(N'[dbo].[DimPolicy]'))
ALTER TABLE [dbo].[DimPolicy] CHECK CONSTRAINT [FK_DimPolicy_DimClient]
GO

IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DimPolicy_DimPolicyType]') AND parent_object_id = OBJECT_ID(N'[dbo].[DimPolicy]'))
ALTER TABLE [dbo].[DimPolicy]  WITH CHECK ADD  CONSTRAINT [FK_DimPolicy_DimPolicyType] FOREIGN KEY([PolicyTypeKey])
REFERENCES [dbo].[DimPolicyType] ([PolicyTypeKey])
GO

IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DimPolicy_DimPolicyType]') AND parent_object_id = OBJECT_ID(N'[dbo].[DimPolicy]'))
ALTER TABLE [dbo].[DimPolicy] CHECK CONSTRAINT [FK_DimPolicy_DimPolicyType]
GO

IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FactClaims]') AND type in (N'U'))
BEGIN
CREATE TABLE [dbo].[FactClaims](
	[ClaimKey] [int] IDENTITY(1,1) NOT NULL,
	[PolicyKey] [int] NOT NULL,
	[ClaimDate] [date] NOT NULL,	
	[ClaimCodeKey] [nvarchar](4) NOT NULL,
	[TotalAmount] [money] NOT NULL,
 CONSTRAINT [PK_FactClaims] PRIMARY KEY CLUSTERED 
(
	[ClaimKey] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
END
GO

IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FactClaims_DimClaimCode]') AND parent_object_id = OBJECT_ID(N'[dbo].[FactClaims]'))
ALTER TABLE [dbo].[FactClaims]  WITH CHECK ADD  CONSTRAINT [FK_FactClaims_DimClaimCode] FOREIGN KEY([ClaimCodeKey])
REFERENCES [dbo].[DimClaimCode] ([ClaimCodeKey])
GO

IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FactClaims_DimClaimCode]') AND parent_object_id = OBJECT_ID(N'[dbo].[FactClaims]'))
ALTER TABLE [dbo].[FactClaims] CHECK CONSTRAINT [FK_FactClaims_DimClaimCode]
GO

IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FactClaims_DimPolicy]') AND parent_object_id = OBJECT_ID(N'[dbo].[FactClaims]'))
ALTER TABLE [dbo].[FactClaims]  WITH CHECK ADD  CONSTRAINT [FK_FactClaims_DimPolicy] FOREIGN KEY([PolicyKey])
REFERENCES [dbo].[DimPolicy] ([PolicyKey])
GO

IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FactClaims_DimPolicy]') AND parent_object_id = OBJECT_ID(N'[dbo].[FactClaims]'))
ALTER TABLE [dbo].[FactClaims] CHECK CONSTRAINT [FK_FactClaims_DimPolicy]
GO


---
--- Initial data
---

INSERT INTO [dbo].[DimPolicyType]
           ([PolicyTypeKey]
           ,[Description])
     VALUES
           (N'AUTO', N'Automobile'),
           (N'BOAT', N'Boat'),
           (N'HOME', N'Home owner''s insurance'),
           (N'RENT', N'Rent''s insurance')
GO


INSERT INTO [dbo].[DimClaimCode]
           ([ClaimCodeKey]
           ,[Description])
     VALUES
           (N'CCAC', N'Car accident, shared responsibility'),
           (N'CDAF', N'Car accident, driver at fault'),
           (N'HWAT', N'Home coverage, water damage')
GO

exec sp_changedbowner 'sa','REDMOND\mmasson'