--
-- MERGE for Type 1 attributes
--

MERGE INTO [DimClient] AS FACT
USING [DimClient_STAGE] AS SRC 
ON (
	FACT.ClientBusinessKey = SRC.ClientBusinessKey
)
WHEN MATCHED AND FACT.EndDate is NULL	-- update the current record
THEN
	UPDATE SET
		FACT.FirstName = SRC.FirstName
		, FACT.TotalChildren = SRC.TotalChildren
		, FACT.YearlyIncome = SRC.YearlyIncome
		, FACT.MiddleName = SRC.MiddleName
		, FACT.AddressLine1 = SRC.AddressLine1
		, FACT.AddressLine2 = SRC.AddressLine2
;

GO

--
-- Handle Type 2 changes
--

INSERT INTO [DimClient] (ClientBusinessKey, FirstName, MiddleName, 
						 LastName, AddressLine1, AddressLine2, BirthDate, 
						 Gender, YearlyIncome, TotalChildren, DateFirstPolicy, 
						 CommuteDistance, City, StateProvinceCode, 
						 StateProvinceName, CountryRegionCode, PostalCode, 
						 StartDate)
SELECT ClientBusinessKey
	, FirstName
	, MiddleName
	, LastName
	, AddressLine1
	, AddressLine2
	, BirthDate
	, Gender
	, YearlyIncome
	, TotalChildren
	, DateFirstPolicy
	, CommuteDistance
	, City
	, StateProvinceCode
	, StateProvinceName
	, CountryRegionCode
	, PostalCode
	, StartDate
FROM (
	MERGE INTO [DimClient] AS FACT
	USING [DimClient_STAGE] AS SRC 
	ON (
		FACT.ClientBusinessKey = SRC.ClientBusinessKey
	)
	WHEN NOT MATCHED THEN	
		INSERT VALUES(
			SRC.ClientBusinessKey
			, SRC.FirstName
			, SRC.MiddleName
			, SRC.LastName
			, SRC.AddressLine1
			, SRC.AddressLine2
			, SRC.BirthDate
			, SRC.Gender
			, SRC.YearlyIncome
			, SRC.TotalChildren
			, SRC.DateFirstPolicy
			, SRC.CommuteDistance
			, SRC.City
			, SRC.StateProvinceCode
			, SRC.StateProvinceName
			, SRC.CountryRegionCode
			, SRC.PostalCode
			, GETDATE()			-- StartDate
			, NULL				-- EndDate
		)
	WHEN MATCHED AND FACT.EndDate is NULL THEN
		UPDATE SET FACT.EndDate = GETDATE()
	OUTPUT $Action Action_Out
			, SRC.ClientBusinessKey
			, SRC.FirstName
			, SRC.MiddleName
			, SRC.LastName
			, SRC.AddressLine1
			, SRC.AddressLine2
			, SRC.BirthDate
			, SRC.Gender
			, SRC.YearlyIncome
			, SRC.TotalChildren
			, SRC.DateFirstPolicy
			, SRC.CommuteDistance
			, SRC.City
			, SRC.StateProvinceCode
			, SRC.StateProvinceName
			, SRC.CountryRegionCode
			, SRC.PostalCode
			, GETDATE() StartDate
) AS MERGE_OUT
WHERE MERGE_OUT.Action_Out = 'UPDATE'


--
-- Drop Constraints
--

IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DimPolicy_DimClient]') AND parent_object_id = OBJECT_ID(N'[dbo].[DimPolicy]'))
ALTER TABLE [dbo].[DimPolicy] DROP CONSTRAINT [FK_DimPolicy_DimClient]
GO


--
-- Re-set constraint
--

IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DimPolicy_DimClient]') AND parent_object_id = OBJECT_ID(N'[dbo].[DimPolicy]'))
ALTER TABLE [dbo].[DimPolicy]  WITH CHECK ADD  CONSTRAINT [FK_DimPolicy_DimClient] FOREIGN KEY([ClientId])
REFERENCES [dbo].[DimClient] ([ClientId])
GO

IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_DimPolicy_DimClient]') AND parent_object_id = OBJECT_ID(N'[dbo].[DimPolicy]'))
ALTER TABLE [dbo].[DimPolicy] CHECK CONSTRAINT [FK_DimPolicy_DimClient]
GO
